﻿using Exam.ExamManages;
using Exam.Infrastructure.Data.Entity;
using Exam.Infrastructure.Enums;
using Exam.Infrastructure.Extensions;
using Exam.Infrastructure.Web.Extensions;
using Exam.Infrastructure.Web.Utilities;
using Exam.Insfrastructure.Service.Service;
using Exam.Questions;
using Exam.Repository.Sqlsugar.Repositories;
using Exam.Share;
using Exam.Share.ViewResponses.Exam;
using Hotline.Application.Exam.Interface.ExamManages;
using Hotline.Repository.SqlSugar;
using Hotline.Repository.SqlSugar.DataPermissions;
using Hotline.Repository.SqlSugar.Exam.Interfaces.ExamManages;
using Hotline.Repository.SqlSugar.Interface;
using Hotline.Share.Dtos.Questions;
using Hotline.Share.Requests.Exam;
using JiebaNet.Segmenter.Common;
using MapsterMapper;
using SqlSugar;
using XF.Domain.Dependency;
using XF.Domain.Entities;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Application.Exam.QueryExtensions.ExamManages;
using XF.Domain.Exceptions;
using NetTaste;
using Exam.Infrastructure.Validation.Validation;

namespace Hotline.Application.Exam.Service.ExamManages
{
    public class ExamManageService : ApiService<ExamManage, AddExamManageDto, UpdateExamManageDto, HotlineDbContext>, IExamManageService,IScopeDependency
    {
        private readonly IExamManageRepository _repository;
        private readonly IExamQuestionScoreRepository _examQuestionScoreRepository;
        private readonly IUserExamRepository _userExamRepository;
        private readonly IDataPermissionFilterBuilder _dataPermissionFilterBuilder;
        private readonly IServiceProvider _serviceProvider;
        private readonly IMapper _mapper;

        public ExamManageService(IExamManageRepository repository,
            IExamQuestionScoreRepository examQuestionScoreRepository,
            IUserExamRepository userExamRepository,
            IDataPermissionFilterBuilder dataPermissionFilterBuilder, IServiceProvider serviceProvider,
            IMapper mapper) : base(repository, mapper)
        {
            this._repository = repository;
            this._examQuestionScoreRepository = examQuestionScoreRepository;
            this._userExamRepository = userExamRepository;
            _dataPermissionFilterBuilder = dataPermissionFilterBuilder;
            _serviceProvider = serviceProvider;
            this._mapper = mapper;
        }

        #region public method
        public async Task<ExamManageDto> GetAsync(EntityQueryRequest entityQueryRequest)
        {
            var entity = await _repository.GetAsync(entityQueryRequest.Id);

            var questionDto = _mapper.Map<ExamManageDto>(entity);

            if (questionDto != null)
            {
                questionDto.ExamQuestionScoreDtos = await GetExamQuestionScores(entityQueryRequest);

                questionDto.UserExamDtos = await GetUserExams(entityQueryRequest);
            }

            return questionDto;
        }

        public async Task<(int, List<ExamManageViewResponse>)> GetListAsync(ExamManagePagedRequest queryRequest)
        {
            var expression = queryRequest.GetExpression();
            var questionTable = _repository.Queryable().Where(expression);

            var queryable = questionTable.Select( e => new ExamManageViewResponse
            {
                Id =e.Id,
                Code =e.Code,
                Count =e.Count,
                CutoffScore= e.CutoffScore,
                EndTime = e.EndTime,
                ExamStatus = e.ExamStatus,
                ExamType = e.ExamType,
                Mehtod = e.Method,
                Mode = e.Mode,
                Name = e.Name,
                Remark = e.Name,
                SortIndex =e.SortIndex,
                StartTime = e.StartTime,
                TimeSpan = e.TimeSpan,
                TotalScore =e.TotalScore,
            });
                

            var result = await queryable.ToListAsync();


            var total = await queryable.CountAsync();

            return (total, result);
        }

        public async Task<PageViewResponse<ExamManageViewResponse>> GetPagedListAsync(ExamManagePagedRequest queryRequest)
        {
            var expression = queryRequest.GetExpression();
            var questionTable = _repository.Queryable().Where(expression);

            var queryable = questionTable.Select(e => new ExamManageViewResponse
            {
                Id = e.Id,
                Code = e.Code,
                Count = e.Count,
                CutoffScore = e.CutoffScore,
                EndTime = e.EndTime,
                ExamStatus = e.ExamStatus,
                ExamType = e.ExamType,
                Mehtod = e.Method,
                Mode = e.Mode,
                Name = e.Name,
                Remark = e.Name,
                SortIndex = e.SortIndex,
                StartTime = e.StartTime,
                TimeSpan = e.TimeSpan,
                TotalScore = e.TotalScore,
            });

            var list = await queryable.ToPageListAsync(queryRequest.PageIndex, queryRequest.PageSize);
            var total = await queryable.CountAsync();

            var result = new ExamManagePageViewResponse
            {
                Items = list,
                Pagination = new Pagination(queryRequest.PageIndex, queryRequest.PageSize, total)
            };

            return result;
        }

        public override async Task<string> AddAsync(AddExamManageDto actionRequest, CancellationToken cancellationToken)
        {
            CalcuteTotalScore(actionRequest);

            var id = await base.AddAsync(actionRequest, cancellationToken);

            ResolveQuestionId(actionRequest, id);            

            await AddExamQuestionScores(actionRequest, cancellationToken);

            await AddUserExam(actionRequest, cancellationToken);

            return id;
        }

        public override async Task UpdateAsync(UpdateExamManageDto actionRequest, CancellationToken cancellationToken)
        {
            CalcuteTotalScore(actionRequest);

            await base.UpdateAsync(actionRequest, cancellationToken);

            ResolveQuestionId(actionRequest, actionRequest.Id);

            await ModifyExamQuestionScores(actionRequest, cancellationToken);

            await ModifyUserExam(actionRequest, cancellationToken);
        }

        public override async Task DeleteAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await base.DeleteAsync(entityQueryRequest, cancellationToken);

            var tmpEntityQueryRequest = ExpressionableUtility.CreateExpression<ExamQuestionScore>()
                .AndIF(entityQueryRequest.Id.IsNotNullOrEmpty(), x => x.ExamManageId == entityQueryRequest.Id)
                .AndIF(entityQueryRequest.Ids.IsNotNull(), x => entityQueryRequest.Ids.Contains(x.ExamManageId))
                .ToEntityQueryRequest<ExamQuestionScore>();

            await DeleteExamQuestionScores(tmpEntityQueryRequest, cancellationToken);

            tmpEntityQueryRequest = ExpressionableUtility.CreateExpression<UserExam>()
                .AndIF(entityQueryRequest.Id.IsNotNullOrEmpty(), x => x.ExamId == entityQueryRequest.Id)
                .AndIF(entityQueryRequest.Ids.IsNotNull(), x => entityQueryRequest.Ids.Contains(x.ExamId))
                .ToEntityQueryRequest<UserExam>();

            await DeleteUserExam(tmpEntityQueryRequest, cancellationToken);

        }
        #endregion

        #region private method

        private void ResolveQuestionId(AddExamManageDto actionRequest, string id)
        {

            actionRequest.UserExamDtos.ForEach(x => x.ExamId = id);

            actionRequest.ExamQuestionScoreDtos.ForEach(x => x.ExamManageId = id);
        }

        private async Task AddExamQuestionScores(AddExamManageDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.ExamQuestionScoreDtos == null) return;

            var examQuestionScoreDtos = actionRequest.ExamQuestionScoreDtos.Where(x => x.OperationStatus == EEOperationStatus.Add).ToList();

            var examQuestionScores = _mapper.Map<List<ExamQuestionScore>>(examQuestionScoreDtos);

            examQuestionScores.ToInsert();

            await _examQuestionScoreRepository.AddWithValidateAsync(examQuestionScores, cancellationToken);
        }

        private async Task AddUserExam(AddExamManageDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.UserExamDtos == null) return;

            var userExamDtos = actionRequest.UserExamDtos.Where(x => x.OperationStatus == EEOperationStatus.Add).ToList();

            var userExams = new List<UserExam>();
            //_mapper.Map<List<UserExam>>(userExamDtos);

            userExamDtos.ForEach(x =>
            {
                userExams.Add(_mapper.Map<UserExam>(x));
            });

            userExams.ToInsert();

            await _userExamRepository.AddWithValidateAsync(userExams, cancellationToken);

        }
        private async Task UpdateUserExam(UpdateExamManageDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.UserExamDtos == null) return;

            var userExamDtos = actionRequest.UserExamDtos.Where(x => x.OperationStatus == EEOperationStatus.Update).ToList();

            var ids = userExamDtos.Select(x => x.Id);

            var userExams = await _userExamRepository.Queryable().Where(x => ids.Contains(x.Id)).ToListAsync();

            userExams = _mapper.Map<List<UpdateUserExamDto>, List<UserExam>>(userExamDtos, userExams);

            userExams.ToUpdate();

            await _userExamRepository.UpdateWithValidateAsync(userExams, cancellationToken);
        }
        private async Task UpdateExamQuestionScores(UpdateExamManageDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.ExamQuestionScoreDtos == null) return;

            var examQuestionScoreDtos = actionRequest.ExamQuestionScoreDtos.Where(x => x.OperationStatus == EEOperationStatus.Update).ToList();

            var ids = examQuestionScoreDtos.Select(x => x.Id);

            var examQuestionScores = await _examQuestionScoreRepository.Queryable().Where(x => ids.Contains(x.Id)).ToListAsync();

            examQuestionScores = _mapper.Map<List<UpdateExamQuestionScoreDto>, List<ExamQuestionScore>>(examQuestionScoreDtos, examQuestionScores);

            examQuestionScores.ToUpdate();

            await _examQuestionScoreRepository.UpdateWithValidateAsync(examQuestionScores, cancellationToken);
        }
    
        private async Task DeleteUserExam(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _userExamRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }
        private async Task DeleteExamQuestionScores(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _examQuestionScoreRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }

        private async Task ModifyUserExam(UpdateExamManageDto actionRequest, CancellationToken cancellationToken)
        {
            await AddUserExam(actionRequest, cancellationToken);

            await UpdateUserExam(actionRequest, cancellationToken);

            var questionOptionsDtos = actionRequest.UserExamDtos.Where(x => x.OperationStatus == EEOperationStatus.Delete);
            var ids = questionOptionsDtos.Select(m => m.Id);
            EntityQueryRequest entityQueryRequest = ResovleDelete<UserExam>(ids);

            await DeleteUserExam(entityQueryRequest, cancellationToken);

        }

        private async Task ModifyExamQuestionScores(UpdateExamManageDto actionRequest, CancellationToken cancellationToken)
        {
            await AddExamQuestionScores(actionRequest, cancellationToken);

            await UpdateExamQuestionScores(actionRequest, cancellationToken);

            var examQuestionScoreDtos = actionRequest.ExamQuestionScoreDtos.Where(x => x.OperationStatus == EEOperationStatus.Delete);
            var ids = examQuestionScoreDtos.Select(m => m.Id);
            EntityQueryRequest entityQueryRequest = ResovleDelete<ExamQuestionScore>(ids);

            await DeleteExamQuestionScores(entityQueryRequest, cancellationToken);
        }

        private EntityQueryRequest ResovleDelete<T>(IEnumerable<string> ids) where T : class, IEntity<string>, new()
        {
            Expressionable<T> expressionable = ExpressionableUtility.CreateExpression<T>();
            expressionable.AndIF(ids.Any(), x => ids.Contains(x.Id));

            var entityQueryRequest = new EntityQueryRequest
            {
                Expression = ids.Any() ? expressionable.ToExpression() : null
            };
            return entityQueryRequest;
        }

        private async Task<List<UserExamDto>> GetUserExams(EntityQueryRequest entityQueryRequest)
        {
            var userExams = _userExamRepository.Queryable().Where(x => x.ExamId == entityQueryRequest.Id);


            var userExamDtos = userExams.Select(q => new UserExamDto
            {
                Id = q.Id,
                ExamId = q.ExamId,
            });

            return await userExamDtos.ToListAsync();
        }

        private async Task<List<ExamQuestionScoreDto>> GetExamQuestionScores(EntityQueryRequest entityQueryRequest)
        {
            var examQuestionScores = _examQuestionScoreRepository.Queryable().Where(x => x.ExamManageId == entityQueryRequest.Id);


            var examQuestionScoreDtos = examQuestionScores.Select(q => new ExamQuestionScoreDto
            {
                Id = q.Id,
                ExamManageId = q.ExamManageId,
                Score = q.Score
            });

            return await examQuestionScoreDtos.ToListAsync();
        }
        private void CalcuteTotalScore(AddExamManageDto actionRequest)
        {
            var examQuestionScoreDtos = actionRequest.ExamQuestionScoreDtos.Where(x => x.OperationStatus != EEOperationStatus.Delete);

            var totalScore = examQuestionScoreDtos.Sum(x => x.Count * x.Score);
            if(totalScore != actionRequest.TotalScore)
            {
                throw new UserFriendlyException(ErrorMessage.ServiceError,string.Format(ErrorMessage.IsNotEqual, "试题分数总和",typeof(AddExamManageDto).GetDescription(nameof(AddExamManageDto.TotalScore))));
            }
        }
        #endregion
    }
}
