﻿using DocumentFormat.OpenXml.Office2010.Excel;
using Exam.Application.Interface.TestPapers;
using Exam.Infrastructure.Data.Entity;
using Exam.Infrastructure.Enums;
using Exam.Infrastructure.Extensions;
using Exam.Infrastructure.Web.Extensions;
using Exam.Infrastructure.Web.Utilities;
using Exam.Insfrastructure.Service.Service;
using Exam.Questions;
using Exam.Repository.Sqlsugar.Repositories;
using Exam.Share.ViewResponses.TestPaper;
using Exam.TestPapers;
using Hotline.Application.Exam.QueryExtensions.TestPapers;
using Hotline.Repository.SqlSugar;
using Hotline.Repository.SqlSugar.DataPermissions;
using Hotline.Repository.SqlSugar.Exam.Interfaces.TestPapers;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Share.Dtos.TestPapers;
using Hotline.Share.Requests.Exam;
using Hotline.Share.Tools;
using Hotline.Share.ViewResponses.Exam;
using JiebaNet.Segmenter.Common;
using MapsterMapper;
using SqlSugar;
using XF.Domain.Dependency;

namespace Hotline.Application.Exam.Service.TestPapers
{
    public class TestPaperService : ApiService<TestPaper, AddTestPaperDto, UpdateTestPaperDto, HotlineDbContext>, ITestPaperService, IScopeDependency
    {
        private readonly ITestPaperRepository _repository;
        private readonly ITestPaperItemRepository _testPaperItemRepository;
        private readonly ITestPaperRuleRepository _testPaperRuleRepository;
        private readonly ITestPaperRuleTagRepository _testPaperRuleTagRepository;
        private readonly IDataPermissionFilterBuilder _dataPermissionFilterBuilder;
        private readonly IServiceProvider _serviceProvider;
        private readonly IMapper _mapper;

        public TestPaperService(ITestPaperRepository repository,
            ITestPaperItemRepository testPaperItemRepository,
            ITestPaperRuleRepository testPaperRuleRepository,
            ITestPaperRuleTagRepository testPaperRuleTagRepository,
            IDataPermissionFilterBuilder dataPermissionFilterBuilder, IServiceProvider serviceProvider,
            IMapper mapper) : base(repository, mapper)
        {
            _repository = repository;
            _testPaperItemRepository = testPaperItemRepository;
            _testPaperRuleRepository = testPaperRuleRepository;
            _testPaperRuleTagRepository = testPaperRuleTagRepository;
            _dataPermissionFilterBuilder = dataPermissionFilterBuilder;
            _serviceProvider = serviceProvider;
            _mapper = mapper;
        }

        #region public method
        public async Task<TestPaperDto> GetAsync(EntityQueryRequest entityQueryRequest)
        {
            var entity = await _repository.GetAsync(entityQueryRequest.Id);

            var testPaperDto = _mapper.Map<TestPaperDto>(entity);

            testPaperDto.TestPaperItemDtos = await GetTestPaperItemDtos(entityQueryRequest);

            testPaperDto.TestPaperRuleDtos = await GetTestPaperRuleDtos(entityQueryRequest);

            return testPaperDto;
        }

        public async Task<(int, List<TestPaperViewResponse>)> GetListAsync(TestPaperPagedRequest queryRequest)
        {
            ISugarQueryable<TestPaperViewResponse> queryable = QueryResult(queryRequest);

            var result = await queryable.ToListAsync();

            var total = await queryable.CountAsync();

            return (total, result);
        }

        public async Task<PageViewResponse<TestPaperViewResponse>> GetPagedListAsync(TestPaperPagedRequest queryRequest)
        {
            ISugarQueryable<TestPaperViewResponse> queryable = QueryResult(queryRequest);

            var list = await queryable.ToPageListAsync(queryRequest.PageIndex, queryRequest.PageSize);
            var total = await queryable.CountAsync();

            var result = new TestPaperPageViewResponse
            {
                Items = list,
                Pagination = new Pagination(queryRequest.PageIndex, queryRequest.PageSize, total)
            };

            return result;
        }

        /// <summary>
        /// 新增试卷
        /// </summary>
        /// <param name="actionRequest"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public override async Task<string> AddAsync(AddTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            var id = await base.AddAsync(actionRequest, cancellationToken);

            ResolveTestPaperId(actionRequest, id);

            await AddTestPaperItems(actionRequest, cancellationToken);

            await AddTestPaperRules(actionRequest, cancellationToken);

            await AddTestPaperRuleTags(actionRequest, cancellationToken);

            return id;
        }

        /// <summary>
        /// 修改试卷
        /// </summary>
        /// <param name="actionRequest"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public override async Task UpdateAsync(UpdateTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            await base.UpdateAsync(actionRequest, cancellationToken);

            ResolveTestPaperId(actionRequest, actionRequest.Id);

            await ModifyTestPaperItems(actionRequest, cancellationToken);

            await ModifyTestPaperRules(actionRequest, cancellationToken);

            await ModifyTestPaperRuleTags(actionRequest, cancellationToken);
        }

        /// <summary>
        /// 删除试卷
        /// </summary>
        /// <param name="entityQueryRequest"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public override async Task DeleteAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            var testPaperRuleTags = await GetTestPaperRuleTags(entityQueryRequest);

            await base.DeleteAsync(entityQueryRequest, cancellationToken);

            var tempEntityQueryRequest = ExpressionableUtility.CreateExpression<TestPaperItem>()
               .AndIF(entityQueryRequest.Id.IsNotNullOrEmpty(), x => x.TestPaperId == entityQueryRequest.Id)
               .AndIF(entityQueryRequest.Ids.IsNotNull(), x => entityQueryRequest.Ids.Contains(x.TestPaperId)).ToEntityQueryRequest<TestPaperItem>();

            await DeleteTestPaperItems(tempEntityQueryRequest, cancellationToken);

            tempEntityQueryRequest = ExpressionableUtility.CreateExpression<TestPaperRule>()
                .AndIF(entityQueryRequest.Id.IsNotNullOrEmpty(), x => x.TestPaperId == entityQueryRequest.Id)
                .AndIF(entityQueryRequest.Ids.IsNotNull(), x => entityQueryRequest.Ids.Contains(x.TestPaperId)).ToEntityQueryRequest<TestPaperRule>();


            await DeleteTestPaperRules(tempEntityQueryRequest, cancellationToken);


            var ids = testPaperRuleTags.Select(x => x.Id).ToList();
            tempEntityQueryRequest = ExpressionableUtility.CreateExpression<TestPaperRuleTag>()
                .AndIF(ids.IsNotNull(), x => ids.Contains(x.Id)).ToEntityQueryRequest<TestPaperRuleTag>();

            await DeleteTestPaperRuleTags(tempEntityQueryRequest, cancellationToken);
        }

        /// <summary>
        /// 获取试卷试题数量
        /// </summary>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        public async Task<List<TestPaperQuestionCountViewResponse>> GetTestPaperQuestionCount(TestPaperQuestionCountRequest testPaperQuestionCountRequest)
        {
            var expression = testPaperQuestionCountRequest.GetTestPaperCountExpression();
            var testPaper = await _repository.Queryable().Where(expression).FirstAsync();

            var testPaperQuestionCountViewResponses = new List<TestPaperQuestionCountViewResponse>();
            if (testPaper != null)
            {
                if(testPaper.Mode == Share.Enums.Exams.EExamMode.Random)
                {
                    testPaperQuestionCountViewResponses = await CalcuteRandomQuestionCount(testPaperQuestionCountRequest);
                }
                else if(testPaper.Mode == Share.Enums.Exams.EExamMode.Manual)
                {
                    testPaperQuestionCountViewResponses = await CalcuteManualQuestionCount(testPaperQuestionCountRequest);
                }
            }
            return testPaperQuestionCountViewResponses;
        }
        #endregion

        #region private method
        /// <summary>
        /// 获取查询结果
        /// </summary>
        /// <param name="queryRequest"></param>
        /// <returns></returns>
        private ISugarQueryable<TestPaperViewResponse> QueryResult(TestPaperPagedRequest queryRequest)
        {
            var expression = queryRequest.GetExpression();
            var query = _repository.Queryable().Where(expression);
            var querable = query.OrderBy(o => o.SortIndex).Select(m => new TestPaperViewResponse
            {
                Name = m.Name,
                ExamType = m.ExamType,
                Mode = m.Mode,
                Code = m.Code,
                Id = m.Id,
                Status = m.Status,
                SortIndex = m.SortIndex,
                Remark = m.Remark,
            });
            return querable;
        }



        private async Task AddTestPaperRules(AddTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperRuleDtos == null) return;

            if (actionRequest.Mode == Share.Enums.Exams.EExamMode.Manual) return;

            var testPaperRuleDtos = actionRequest.TestPaperRuleDtos.Where(x => x.OperationStatus == EEOperationStatus.Add).ToList();
            var testPaperRules = _mapper.Map<List<TestPaperRule>>(testPaperRuleDtos);

            testPaperRules.ToInsert();

            await _testPaperRuleRepository.AddWithValidateAsync(testPaperRules, cancellationToken);

        }

        private async Task AddTestPaperItems(AddTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperItemDtos == null) return;

            var testPaperItemDtos = actionRequest.TestPaperItemDtos.Where(x => x.OperationStatus == EEOperationStatus.Add).ToList();
            var testPaperItems = _mapper.Map<List<TestPaperItem>>(testPaperItemDtos);

            testPaperItems.ToInsert();

            await _testPaperItemRepository.AddWithValidateAsync(testPaperItems, cancellationToken);
        }


        private async Task AddTestPaperRuleTags(AddTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperRuleDtos == null) return;

            var testPaperRuleDtos = actionRequest.TestPaperRuleDtos.Where(x => x.OperationStatus == EEOperationStatus.Add);

            var testPaperRuleTagDtos = testPaperRuleDtos.Select(x => x.TestPaperRuleTagDto);

            var testPaperRuleTags = _mapper.Map<List<TestPaperRuleTag>>(testPaperRuleTagDtos);

            testPaperRuleTags.ToInsert();

            await _testPaperRuleTagRepository.AddWithValidateAsync(testPaperRuleTags, cancellationToken);
        }

        private async Task ModifyTestPaperRules(UpdateTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperRuleDtos == null) return;

            if (actionRequest.Mode == Share.Enums.Exams.EExamMode.Manual) return;

            await AddTestPaperRules(actionRequest, cancellationToken);

            await UpdateTestPaperRules(actionRequest, cancellationToken);

            var entityQueryRequest = ExpressionableUtility.CreateExpression<TestPaperRule>()
                .AndIF(actionRequest.Id.IsNotNullOrEmpty(), x => x.TestPaperId == actionRequest.Id).ToEntityQueryRequest<TestPaperRule>();

            await DeleteTestPaperRules(entityQueryRequest, cancellationToken);
        }

        private async Task DeleteTestPaperRules(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _testPaperRuleRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }

        private async Task UpdateTestPaperRules(UpdateTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperRuleDtos == null) return;

            var testPaperRuleDtos = actionRequest.TestPaperRuleDtos.Where(x => x.OperationStatus == EEOperationStatus.Update).ToList();

            var ids = testPaperRuleDtos.Select(x => x.Id);

            var testPaperRules = await _testPaperRuleRepository.Queryable().Where(x => ids.Contains(x.TestPaperId)).ToListAsync();

            var entities = new List<TestPaperRule>();

            foreach (var testPaperRuleDto in testPaperRuleDtos)
            {
                var entity = testPaperRules.FirstOrDefault(x => x.Id == testPaperRuleDto.Id);

                entity = _mapper.Map(testPaperRuleDto, entity);

                entity.TestPaperId = actionRequest.Id;
            }

            entities.ToUpdate();

            await _testPaperRuleRepository.UpdateWithValidateAsync(entities, cancellationToken);

        }

        private async Task ModifyTestPaperRuleTags(UpdateTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperRuleDtos == null) return;

            await AddTestPaperRuleTags(actionRequest, cancellationToken);

            await UpdateTestPaperRuleTags(actionRequest, cancellationToken);

            var ids = actionRequest.TestPaperRuleDtos.Where(x => x.OperationStatus == EEOperationStatus.Delete).Select(x => x.Id).ToList();

            var entityQueryRequest = ExpressionableUtility.CreateExpression<TestPaperRuleTag>()
                .AndIF(ids.IsNullOrEmpty(), x => ids.Contains(x.TestPaperRuleId)).ToEntityQueryRequest<TestPaperRuleTag>();

            await DeleteTestPaperRuleTags(entityQueryRequest, cancellationToken);
        }

        private async Task DeleteTestPaperRuleTags(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _testPaperRuleTagRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }

        private async Task UpdateTestPaperRuleTags(UpdateTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperRuleDtos == null) return;

            var ruleIds = actionRequest.TestPaperRuleDtos.Where(x => x.OperationStatus == EEOperationStatus.Update).Select(x => x.Id).ToList();

            var testPaperRuleTags = await _testPaperRuleTagRepository.Queryable().Where(x => ruleIds.Contains(x.TestPaperRuleId)).ToListAsync();

            var entities = new List<TestPaperRuleTag>();

            foreach (var testPaperRuleDto in actionRequest.TestPaperRuleDtos)
            {
                var entity = testPaperRuleTags.FirstOrDefault(x => x.TestPaperRuleId == testPaperRuleDto.Id);

                entity.TagId = testPaperRuleDto.TagId;

                entity.TestPaperRuleId = testPaperRuleDto.Id;
            }

            entities.ToUpdate();

            await _testPaperRuleTagRepository.UpdateWithValidateAsync(entities, cancellationToken);
        }

        private async Task ModifyTestPaperItems(UpdateTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperItemDtos == null) return;

            await AddTestPaperItems(actionRequest, cancellationToken);

            await UpdateTestPaperItems(actionRequest, cancellationToken);

            var entityQueryRequest = ExpressionableUtility.CreateExpression<TestPaperItem>()
                .AndIF(actionRequest.Id.IsNotNullOrEmpty(), x => x.TestPaperId == actionRequest.Id).ToEntityQueryRequest<TestPaperItem>();

            await DeleteTestPaperItems(entityQueryRequest, cancellationToken);
        }

        private async Task DeleteTestPaperItems(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            await _testPaperItemRepository.DeleteWithValidateAsync(entityQueryRequest, cancellationToken);
        }

        private async Task UpdateTestPaperItems(UpdateTestPaperDto actionRequest, CancellationToken cancellationToken)
        {
            if (actionRequest.TestPaperItemDtos == null) return;

            var testPaperItemDtos = actionRequest.TestPaperItemDtos.Where(x => x.OperationStatus == EEOperationStatus.Update).ToList();

            var ids = testPaperItemDtos.Select(x => x.Id);

            var testPaperItems = await _testPaperItemRepository.Queryable().Where(x => ids.Contains(x.TestPaperId)).ToListAsync();

            var entities = new List<TestPaperItem>();

            foreach (var testPaperItemDto in testPaperItemDtos)
            {
                var entity = testPaperItems.FirstOrDefault(x => x.Id == testPaperItemDto.Id);

                entity = _mapper.Map(testPaperItemDto, entity);

                entity.TestPaperId = actionRequest.Id;
            }

            entities.ToUpdate();

            await _testPaperItemRepository.UpdateWithValidateAsync(entities, cancellationToken);
        }

        /// <summary>
        /// 获取试卷规则
        /// </summary>
        /// <param name="entityQueryRequest"></param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        private async Task<List<TestPaperRuleDto>> GetTestPaperRuleDtos(EntityQueryRequest entityQueryRequest)
        {
            var testPaperRuleTable = _testPaperRuleRepository.Queryable().Where(x => x.TestPaperId == entityQueryRequest.Id);

            var testPaperRuleTags = await GetTestPaperRuleTags(entityQueryRequest);

            var queryable = testPaperRuleTable.Select(t => new TestPaperRuleDto
            {
                Id = t.Id,
                TestPaperId = t.TestPaperId,
                Count = t.Count,
                DifficultyLevel = t.DifficultyLevel,
                SortIndex = t.SortIndex,
                QuestionType = t.QuestionType,
            });

            var testPaperRuleTagDtos = _mapper.Map<List<TestPaperRuleTagDto>>(testPaperRuleTags);
            var testPaperRuleList = await queryable.ToListAsync();
            testPaperRuleList.ForEach(x => x.TestPaperRuleTagDto = testPaperRuleTagDtos.FirstOrDefault(n => n.TestPaperRuleId == x.Id));

            return testPaperRuleList;

        }

        private async Task<List<TestPaperItemDto>> GetTestPaperItemDtos(EntityQueryRequest entityQueryRequest)
        {
            var testPaperRuleTable = _testPaperItemRepository.Queryable().Where(x => x.TestPaperId == entityQueryRequest.Id);

            var questionTable = new ExamRepository<Question>(_testPaperItemRepository.UOW, _dataPermissionFilterBuilder, _serviceProvider).Queryable();

            var queryable = testPaperRuleTable.InnerJoin(questionTable, (t, q) => t.QuestionId == q.Id).Select((t, q) => new TestPaperItemDto
            {
                Id = t.Id,
                QuestionId = t.QuestionId,
                DifficultyLevel = q.DifficultyLevel,
                Title = q.Title,
                TestPaperId = t.TestPaperId,
                QuestionType = q.QuestionType
            });

            return await queryable.ToListAsync();

        }

        private void ResolveTestPaperId(AddTestPaperDto actionRequest, string id)
        {
            actionRequest.TestPaperRuleDtos.ForEach(x =>
            {
                x.TestPaperId = id;
                x.TestPaperRuleTagDto = new TestPaperRuleTagDto
                {
                    TagId = x.TagId,
                    TestPaperRuleId = id
                };
            });

            actionRequest.TestPaperItemDtos.ForEach(x => x.TestPaperId = id);
        }

        private async Task<List<TestPaperRuleTag>> GetTestPaperRuleTags(EntityQueryRequest entityQueryRequest)
        {
            var expression = entityQueryRequest.GetTestPaperRuleExpression();
            var testPageRuleTable = _testPaperRuleRepository.Queryable().Where(expression);
            var testPageRuleTagTable = _testPaperRuleTagRepository.Queryable();

            var query = testPageRuleTagTable.InnerJoin(testPageRuleTable, (t, r) => t.TestPaperRuleId == r.Id).Select((t, r) => t);

            return await query.ToListAsync();
        }

        private async Task<List<TestPaperQuestionCountViewResponse>> CalcuteRandomQuestionCount(TestPaperQuestionCountRequest testPaperQuestionCountRequest)
        {
            var testPaperQuestionCountViewResponses = new List<TestPaperQuestionCountViewResponse>();

            var expression = testPaperQuestionCountRequest.GetTestPaperRuleExpression();

            var testPaperRules = await _testPaperRuleRepository.Queryable().Where(expression).ToListAsync();

            if (testPaperRules != null)
            {
                testPaperRules.ForEach(x => testPaperQuestionCountViewResponses.Add(new TestPaperQuestionCountViewResponse
                {
                    QuestionType = x.QuestionType,
                    Count = x.Count
                }));
            }

            return testPaperQuestionCountViewResponses;
        }

        private async Task<List<TestPaperQuestionCountViewResponse>> CalcuteManualQuestionCount(TestPaperQuestionCountRequest testPaperQuestionCountRequest)
        {
            var testPaperQuestionCountViewResponses = new List<TestPaperQuestionCountViewResponse>();

            var expression = testPaperQuestionCountRequest.GetTestPaperItemsExpression();

            var testPaperItemTable =  _testPaperItemRepository.Queryable().Where(expression);
            var questionTable = new ExamRepository<Question>(_testPaperItemRepository.UOW, _dataPermissionFilterBuilder, _serviceProvider).Queryable();

            testPaperQuestionCountViewResponses = await testPaperItemTable.InnerJoin(questionTable, (t, q) => t.QuestionId == q.Id)
                .GroupBy((t, q) => q.QuestionType).Select((t, q) => new TestPaperQuestionCountViewResponse
                {
                    QuestionType = q.QuestionType,
                    Count = SqlFunc.AggregateCount(t.Id)
                }).ToListAsync();           


            return testPaperQuestionCountViewResponses;
        }
        #endregion
    }
}
